/*!
 * node-tair - benchmark/bench.js
 *
 * Copyright(c) 2013 fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 * MIT Licensed
 *
 * $ node bench.js [get|put|prefixGets|prefixSets] [bytes=100] [concurrency=10] [requests=10000]
 */

"use strict";

/**
 * Module dependencies.
 */

var ab = require('ab');
var Tair = require('../');

// var tair = Tair({
//   dataId: 'market2-daily',
//   diamond: 'commonconfig.taobao.net:8080',
//   namespace: 171,
// });

var tair = Tair({
  servers: [{host: '127.0.0.1', port: 5198}],
  group: 'testgroup',
});

var method = process.argv[2] || 'get';
var bytes = parseInt(process.argv[3], 10) || 100;

var options = {
  concurrency: parseInt(process.argv[4], 10) || 10,
  requests: parseInt(process.argv[5], 10) || 10000,
};

var benchName = method + 'Bytes(' + bytes + ')';

console.log('$ node bench.js %s [bytes=%d] [concurrency=%d] [requests=%d]\n%s;\n',
  method, bytes, options.concurrency, options.requests, benchName);

function setBytes(size) {
  var buf = new Buffer(size);
  var i = 0;
  var key = 'setBytes' + size + '-';
  ab.run(function (callback) {
    tair.set(key + i++, buf, function (err, success, res) {
      callback(err, success, {
        reqSize: res.reqSize,
        resSize: res.bodySize
      });
    });
  }, options).on('end', function () {
    tair.close();
  });
}

function prefixSetsBytes(size) {
  var buf = new Buffer(size);
  var i = 0;
  var key = 'prefixSetsBytes' + size + '-';
  ab.run(function (callback) {
    tair.set(key + i++, buf, function (err) {
      callback(err, true);
    });
  }, options).on('end', function () {
    tair.close();
  });
}

function getBytes(size) {
  var i = 0;
  var key = 'setBytes' + size + '-';
  ab.run(function (callback) {
    var k = key + i++;
    tair.get(k, function (err, data, res) {
      // console.log(k, data)
      callback(err, data && data.length === size, {
        reqSize: res.reqSize,
        resSize: res.bodySize
      });
    });
  }, options).on('end', function () {
    tair.close();
  });
}

function prefixGetsBytes(size) {
  var i = 0;
  var key = 'prefixSetsBytes' + size + '-';
  var fields = ['user_gender', 'mytaobao_user_interest_group', 'user_interest_group', 'ali_num'];
  ab.run(function (callback) {
    tair.prefixGets(key + i++, fields, function (err, data, res) {
      callback(err, data ? true : false, {
        reqSize: res.reqSize,
        resSize: res.bodySize
      });
    });
  }, options).on('end', function () {
    tair.close();
  });
}

tair.on('connect', function () {
  eval(benchName);
});
