/*!
 * node-tair - benchmark/get.js
 *
 * Copyright(c) 2013 fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 * MIT Licensed
 */

"use strict";

/**
 * Module dependencies.
 */

var fs = require('fs');
var path = require('path');
var Benchmark = require('benchmark');
var Transcoder = require('../../lib/packets/transcoder');
var packets = require('../../lib/packets');
var ByteBuffer = require('byte');

var fixtures = path.join(path.dirname(path.dirname(__dirname)), 'test', 'fixtures');

var transcoder = new Transcoder();
var pkey = '10241024102410241024';
var options = {
  namespace: 171
};

function createGetPacket(pkey, options) {
  options = options || {};
  options.namespace = options.namespace || 0;

  var get = new packets.Get(pkey, transcoder, options);
  return get;
}

var GET = new packets.Get('foo', transcoder);

var datas = {
  100: fs.readFileSync(path.join(fixtures, 'get_100bytes_response.bin')),
  500: fs.readFileSync(path.join(fixtures, 'get_500bytes_response.bin')),
  1000: fs.readFileSync(path.join(fixtures, 'get_1000bytes_response.bin')),
};
function decodeGetPacket(size) {
  return GET.decode(ByteBuffer.wrap(datas[size]));
}

// console.log(decodeGetPacket(100));
// console.log(decodeGetPacket(500));
// console.log(decodeGetPacket(1000));

// var n = 1000000;
// var start = Date.now();
// for (var i = 0; i < n; i++) {
//   createGetPacket(pkey, options);
// }
// var use = Date.now() - start;
// var sec = use / 1000;
// console.log('100w createGetPacket: %s ops/sec, %s sec, avg rt %s ms',
//   (n / sec).toFixed(0), sec.toFixed(3), (use / n).toFixed(3));

var suite = new Benchmark.Suite();
suite

.add('createGetPacket()', function () {
  createGetPacket(pkey, options);
})
.add('decodeGetPacket(100)', function () {
  decodeGetPacket(100);
})
.add('decodeGetPacket(500)', function () {
  decodeGetPacket(500);
})
.add('decodeGetPacket(1000)', function () {
  decodeGetPacket(1000);
})

.add('createGetPacket() and decodeGetPacket(100)', function () {
  createGetPacket(pkey, options);
  decodeGetPacket(100);
})
.add('createGetPacket() and decodeGetPacket(500)', function () {
  createGetPacket(pkey, options);
  decodeGetPacket(500);
})
.add('createGetPacket() and decodeGetPacket(1000)', function () {
  createGetPacket(pkey, options);
  decodeGetPacket(1000);
})

// add listeners
.on('cycle', function (event) {
  console.log(String(event.target));
})
.on('complete', function () {
  console.log('Fastest is ' + this.filter('fastest').pluck('name'));
})
.run();
