/*!
 * node-tair - benchmark/prefix_gets.js
 * Copyright(c) 2013 fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 * MIT Licensed
 */

"use strict";

/**
 * Module dependencies.
 */

var fs = require('fs');
var path = require('path');
var Benchmark = require('benchmark');
var Transcoder = require('../../lib/packets/transcoder');
var packets = require('../../lib/packets');
var MixedKey = packets.MixedKey;
var ByteBuffer = require('byte');

var fixtures = path.join(path.dirname(path.dirname(__dirname)), 'test', 'fixtures');

var transcoder = new Transcoder();
var pkey = '10241024102410241024';
var fields = ['user_gender', 'mytaobao_user_interest_group', 'user_interest_group', 'ali_num'];
var options = {
  namespace: 171
};

function createPrefixPacket(pkey, skeys, options) {
  options = options || {};
  options.namespace = options.namespace || 0;

  var keys = [];
  for (var i = 0; i < skeys.length; i++) {
    var key = new MixedKey(transcoder, pkey, skeys[i]);
    keys.push(key);
  }
  var get = new packets.PrefixGet(keys, transcoder, options);
  return get;
}

var datas = {
  100: fs.readFileSync(path.join(fixtures, 'prefix_gets_100bytes_response.bin')),
  500: fs.readFileSync(path.join(fixtures, 'prefix_gets_500bytes_response.bin')),
  1000: fs.readFileSync(path.join(fixtures, 'prefix_gets_1000bytes_response.bin')),
};
var PREFIX_GET = new packets.PrefixGet([new MixedKey(transcoder, pkey, '123')], transcoder);
function decodePacket(size) {
  return PREFIX_GET.decode(ByteBuffer.wrap(datas[size]));
}

// console.log(decodePacket(100));
// console.log(decodePacket(500));
// console.log(decodePacket(1000));

// var n = 1000000;
// var start = Date.now();
// for (var i = 0; i < n; i++) {
//   createPrefixPacket(pkey, fields, options);
// }
// var use = Date.now() - start;
// var sec = use / 1000;
// console.log('100w createPrefixPacket(pkey, fields, options): %s ops/sec, %s sec, avg rt %s ms',
//   (n / sec).toFixed(0), sec.toFixed(3), (use / n).toFixed(3));

var suite = new Benchmark.Suite();
suite

.add('createPrefixPacket(pkey, skeys, options)', function () {
  createPrefixPacket(pkey, fields, options);
})

.add('decodePacket(100)', function () {
  decodePacket(100);
})
.add('decodePacket(500)', function () {
  decodePacket(500);
})
.add('decodePacket(1000)', function () {
  decodePacket(1000);
})

.add('createPrefixPacket() and decodePacket(100)', function () {
  createPrefixPacket(pkey, fields, options);
  decodePacket(100);
})
.add('createPrefixPacket() and decodePacket(500)', function () {
  createPrefixPacket(pkey, fields, options);
  decodePacket(500);
})
.add('createPrefixPacket() and decodePacket(1000)', function () {
  createPrefixPacket(pkey, fields, options);
  decodePacket(1000);
})

// add listeners
.on('cycle', function (event) {
  console.log(String(event.target));
})
.on('complete', function () {
  console.log('Fastest is ' + this.filter('fastest').pluck('name'));
})
.run();
