/*!
 * tair - lib/const.js
 *
 * Copyright(c) 2012 - 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   kate.sf <kate.sf@taobao.com>
 *   fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 */

/**
 * const variables for this module
 */

var ZERO = new Buffer(1);
ZERO[0] = 0;

var META_FLAG = new Buffer(7);
META_FLAG.fill(0);
var DEFAULT_DATA = new Buffer(29);
DEFAULT_DATA.fill(0);
var EMPTY_META = new Buffer(36);
EMPTY_META.fill(0);

module.exports = {
  META_FLAG: META_FLAG,
  DEFAULT_DATA: DEFAULT_DATA,
  EMPTY_META: EMPTY_META,
  ZERO: ZERO,
  MAX_INT32: 0x7FFFFFFF, // 2147483647;

  // packet flag
  TAIR_PACKET_FLAG: 0x6d426454,
  PACKET_HEADER_SIZE: 12,

// packet code

// data_node_hearbeat, only for node-tair
  REQ_DATA_NODE_HEARTBEAT: 9999,

// request
  TAIR_REQ_PUT_PACKET: 1,
  TAIR_REQ_GET_PACKET: 2,
  TAIR_REQ_REMOVE_PACKET: 3,
  TAIR_REQ_PING_PACKET: 6,
  TAIR_REQ_INVALID_PACKET: 23,
  TAIR_REQ_INCDEC_PACKET: 11,
  TAIR_REQ_LOCK_PACKET: 14,
  TAIR_REQ_PREFIX_GETS_PACKET: 29,
  TAIR_REQ_PREFIX_PUTS_PACKET: 24,
  TAIR_REQ_GET_RANGE_PACKET: 18,
  TAIR_RESP_GET_RANGE_PACKET: 19,

  TAIR_RESP_MRETURN_PACKET: 28,

// response
  TAIR_RESP_RETURN_PACKET: 101,
  TAIR_RESP_GET_PACKET: 102,
  TAIR_RESP_INCDEC_PACKET: 105,

// config server
  TAIR_REQ_GET_GROUP_NEW_PACKET: 1002,
  TAIR_RESP_GET_GROUP_NEW_PACKET: 1102,
  TAIR_REQ_QUERY_INFO_PACKET: 1009,
  TAIR_RESP_QUERY_INFO_PACKET: 1106,

// items
  TAIR_REQ_ADDITEMS_PACKET: 1400,
  TAIR_REQ_GETITEMS_PACKET: 1401,
  TAIR_REQ_REMOVEITEMS_PACKET: 1402,
  TAIR_REQ_GETANDREMOVEITEMS_PACKET: 1403,
  TAIR_REQ_GETITEMSCOUNT_PACKET: 1404,
  TAIR_RESP_GETITEMS_PACKET: 1405,

  /*
   *  extend for redis 2[12]xx,
   *  first number 2 means extend packet,
   *  second number 1 means request, 2 means response
   */
//list
  TAIR_REQ_LPOP_PACKET: 2100,
  TAIR_RESP_LPOP_PACKET: 2200,

  TAIR_REQ_LPUSH_PACKET: 2101,
  TAIR_RESP_LPUSH_PACKET: 2201,

  TAIR_REQ_RPOP_PACKET: 2102,
  TAIR_RESP_RPOP_PACKET: 2202,

  TAIR_REQ_RPUSH_PACKET: 2103,
  TAIR_RESP_RPUSH_PACKET: 2203,

  TAIR_REQ_LPUSHX_PACKET: 2104,
  TAIR_RESP_LPUSHX_PACKET: 2204,

  TAIR_REQ_RPUSHX_PACKET: 2105,
  TAIR_RESP_RPUSHX_PACKET: 2205,

  TAIR_REQ_LINDEX_PACKET: 2106,
  TAIR_RESP_LINDEX_PACKET: 2206,

  TAIR_REQ_LTRIM_PACKET: 2128,
  TAIR_RESP_LTRIM_PACKET: 2228,

  TAIR_REQ_LREM_PACKET: 2129,
  TAIR_RESP_LREM_PACKET: 2229,

  TAIR_REQ_LRANGE_PACKET: 2130,
  TAIR_RESP_LRANGE_PACKET: 2230,

  TAIR_REQ_LLEN_PACKET: 2133,
  TAIR_RESP_LLEN_PACKET: 2233,

//hset
  TAIR_REQ_HGETALL_PACKET: 2107,
  TAIR_RESP_HGETALL_PACKET: 2207,

  TAIR_REQ_HINCRBY_PACKET: 2108,
  TAIR_RESP_HINCRBY_PACKET: 2208,

  TAIR_REQ_HMSET_PACKET: 2109,
  TAIR_RESP_HMSET_PACKET: 2209,

  TAIR_REQ_HSET_PACKET: 2110,
  TAIR_RESP_HSET_PACKET: 2210,

  TAIR_REQ_HSETNX_PACKET: 2111,
  TAIR_RESP_HSETNX_PACKET: 2211,

  TAIR_REQ_HGET_PACKET: 2112,
  TAIR_RESP_HGET_PACKET: 2212,

  TAIR_REQ_HMGET_PACKET: 2113,
  TAIR_RESP_HMGET_PACKET: 2213,

  TAIR_REQ_HVALS_PACKET: 2114,
  TAIR_RESP_HVALS_PACKET: 2214,

  TAIR_REQ_HDEL_PACKET: 2115,
  TAIR_RESP_HDEL_PACKET: 2215,

  TAIR_REQ_HLEN_PACKET: 2136,
  TAIR_RESP_HLEN_PACKET: 2236,

//set
  TAIR_REQ_SCARD_PACKET: 2116,
  TAIR_RESP_SCARD_PACKET: 2216,

  TAIR_REQ_SMEMBERS_PACKET: 2117,
  TAIR_RESP_SMEMBERS_PACKET: 2217,

  TAIR_REQ_SADD_PACKET: 2118,
  TAIR_RESP_SADD_PACKET: 2218,

  TAIR_REQ_SPOP_PACKET: 2119,
  TAIR_RESP_SPOP_PACKET: 2219,

//zset
  TAIR_REQ_ZRANGE_PACKET: 2120,
  TAIR_RESP_ZRANGE_PACKET: 2220,

  TAIR_REQ_ZREVRANGE_PACKET: 2121,
  TAIR_RESP_ZREVRANGE_PACKET: 2221,

  TAIR_REQ_ZSCORE_PACKET: 2122,
  TAIR_RESP_ZSCORE_PACKET: 2222,

  TAIR_REQ_ZRANGEBYSCORE_PACKET: 2123,
  TAIR_RESP_ZRANGEBYSCORE_PACKET: 2223,

  TAIR_REQ_ZADD_PACKET: 2124,
  TAIR_RESP_ZADD_PACKET: 2224,

  TAIR_REQ_ZRANK_PACKET: 2125,
  TAIR_RESP_ZRANK_PACKET: 2225,

  TAIR_REQ_ZCARD_PACKET: 2126,
  TAIR_RESP_ZCARD_PACKET: 2226,

  TAIR_REQ_ZREM_PACKET: 2137,
  TAIR_RESP_ZREM_PACKET: 2237,

  TAIR_REQ_ZREMRANGEBYRANK_PACKET: 2138,
  TAIR_RESP_ZREMRANGEBYRANK_PACKET: 2238,

  TAIR_REQ_ZREMRANGEBYSCORE_PACKET: 2139,
  TAIR_RESP_ZREMRANGEBYSCORE_PACKET: 2239,

  TAIR_REQ_ZREVRANK_PACKET: 2140,
  TAIR_RESP_ZREVRANK_PACKET: 2240,

  TAIR_REQ_ZCOUNT_PACKET: 2141,
  TAIR_RESP_ZCOUNT_PACKET: 2241,

  TAIR_REQ_ZINCRBY_PACKET: 2142,
  TAIR_RESP_ZINCRBY_PACKET: 2242,

//common
  TAIR_REQ_EXPIRE_PACKET: 2127,
  TAIR_RESP_EXPIRE_PACKET: 2227,

  TAIR_REQ_EXPIREAT_PACKET: 2131,
  TAIR_RESP_EXPIREAT_PACKET: 2231,

  TAIR_REQ_PERSIST_PACKET: 2132,
  TAIR_RESP_PERSIST_PACKET: 2232,

  TAIR_REQ_TTL_PACKET: 2134,
  TAIR_RESP_TTL_PACKET: 2234,

  TAIR_REQ_TYPE_PACKET: 2135,
  TAIR_RESP_TYPE_PACKET: 2235,

// serialize type
  TAIR_STYPE_INT: 1,
  TAIR_STYPE_STRING: 2,
  TAIR_STYPE_BOOL: 3,
  TAIR_STYPE_LONG: 4,
  TAIR_STYPE_DATE: 5,
  TAIR_STYPE_BYTE: 6,
  TAIR_STYPE_FLOAT: 7,
  TAIR_STYPE_DOUBLE: 8,
  TAIR_STYPE_BYTEARRAY: 9,
  TAIR_STYPE_SERIALIZE: 10,
  TAIR_STYPE_INCDATA: 11,
  TAIR_STYPE_MIXEDKEY: 12,

  TAIR_PACKET_HEADER_SIZE: 16,
  TAIR_PACKET_HEADER_BLPOS: 12, // body length position

// buffer size
  INOUT_BUFFER_SIZE: 8192,
  DEFAULT_TIMEOUT: 2000,
  DEFAULT_CS_CONN_TIMEOUT: 1000,
  DEFAULT_CS_TIMEOUT: 500,
  DEFAULT_WAIT_THREAD: 100,

// etc
  TAIR_DEFAULT_COMPRESSION_THRESHOLD: 8192,
  TAIR_DEFAULT_COMPRESSION_TYPE: 0,
  DEFAULT_CHARSET: "UTF-8",
  TAIR_KEY_MAX_LENTH: 1024, // 1KB
  TAIR_VALUE_MAX_LENGTH: 1000000, // 1,000,000 bytes ~ 1MB

  TAIR_MAX_COUNT: 1024,
  TAIR_MALLOC_MAX: 1024 * 1024, // 1MB

  NAMESPACE_MAX: 1023,

  ITEM_ALL: 65537,

  INVALUD_SERVERLIST_KEY: "invalidate_server",

  Q_AREA_CAPACITY: 1,
  Q_MIG_INFO: 2,
  Q_DATA_SEVER_INFO: 3,
  Q_GROUP_INFO: 4,
  Q_STAT_INFO: 5,

  NOT_CARE_VERSION: 0,
  NOT_CARE_EXPIRE: -1,
  CANCEL_EXPIRE: 0,

///////////////////constant//////////////////
  REQUEST_ENCODE_OK: 0,
  KEYTOLARGE: 1,
  VALUETOLARGE: 2,
  SERIALIZEERROR: 3,
  DATALENTOOLONG: 4,

  DATA_ENTRY_MAX_SIZE: 1024 * 1024,
  LIST_MAX_LEN: 8192,

  PREFIX_KEY_OFFSET: 22,
  PREFIX_KEY_MASK: 0x3FFFFF,

  CMD_RANGE_ALL: 1,
  CMD_RANGE_VALUE_ONLY: 2,
  CMD_RANGE_KEY_ONLY: 3,
  CMD_RANGE_ALL_REVERSE: 4,
  CMD_RANGE_VALUE_ONLY_REVERSE: 5,
  CMD_RANGE_KEY_ONLY_REVERSE: 6,
  CMD_DEL_RANGE: 7,
  CMD_DEL_RANGE_REVERSE: 8,
  DEFAULT_RANGE_LIMIT: 1000,
};
