/**!
 * node-tair - lib/packets/base.js
 *
 * Copyright(c) 2012 - 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   fengmk2 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var debug = require('debug')('tair:packets:base');
var ByteBuffer = require('byte');
var cons = require('../const');
var utils = require('../utils');

var META_FLAG = cons.META_FLAG;
var DEFAULT_DATA = cons.DEFAULT_DATA;

function BasePacket(code, transcoder, isRead) {
  this.code = code;
  this.transcoder = transcoder;
  this.isRead = isRead === false ? false : true;
}

BasePacket.ResponseCode = cons.TAIR_RESP_RETURN_PACKET;

BasePacket.prototype.encode = function (reqId) {
  // var bytes = ByteBuffer.allocate();
  // // request packet format: | header (16 bytes) | body |
  // // header: | flag: 6d 42 64 54 | request id (4 bytes) | packet code (4 bytes) |
  // buf.writeInt32(cons.TAIR_PACKET_FLAG); // packet flag
  // buf.writeInt32(this.id); // request id
  // // | packet code (4 bytes) | body length (4 bytes) | body |
  // buf.writeInt32(this.code); // 12
  // return bytes.array();
  throw new Error('Not implementation.');
};

BasePacket.prototype.fillMetaFlag = function (out, flag) {
  flag = flag || 0;
  // 7 + 29 = 36 bytes
  out.put(META_FLAG);
  if (flag !== 0) {
    // put flag implicitly
    // out.write(bytes.position() - 13, (byte)flag);
    DEFAULT_DATA[16] = flag; // 29 - 13 = 16
    out.put(DEFAULT_DATA);
    DEFAULT_DATA[16] = 0;
  } else {
    out.put(DEFAULT_DATA);
  }
};

BasePacket.prototype.decode = function (buf) {
  var res = {
    version: buf.getInt(),
    code: buf.getInt(),
    message: buf.getString()
  };
  return res;
};

BasePacket.pack = BasePacket.packError = function (res) {
  var bytes = ByteBuffer.allocate(12 + (res.message && res.message.length || 0));
  bytes.putInt(res.version || 0);
  bytes.putInt(res.value || 0);
  bytes.putString(res.message); // empty message
  return bytes.array();
};


module.exports = BasePacket;
