/*!
 * node-tair - lib/packets/data_node_heartbeat.js
 *
 * Copyright(c) 2012 - 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   fengmk2 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 *   sunfang1cn <kate.sf@taobao.com>
 */

"use strict";

/**
 * Module dependencies.
 */

var debug = require('debug')('tair:packets:data_node_heartbeat');
var util = require('util');
var ByteBuffer = require('byte');
var cons = require('../const');
var BasePacket = require('./base');

/**
 * DataNode Heartbeat packet
 *
 * @param {String} group
 * @param {String} host
 * @param {Number} port
 */
function DataNodeHeartBeat(group, host, port) {
  BasePacket.call(this, cons.REQ_DATA_NODE_HEARTBEAT, false);

  this.group = group;
  this.host = host;
  this.port = port;
}

util.inherits(DataNodeHeartBeat, BasePacket);

DataNodeHeartBeat.prototype.encode = function (reqId) {
  var groupBytes = new Buffer(this.group);
  var hostBytes = new Buffer(this.host);
  // format: | group | host | port (4 bytes)
  var bodyLength = 4 + groupBytes.length + 4 + hostBytes.length + 4;

  var totalLength = cons.PACKET_HEADER_SIZE + 4 + bodyLength;
  var bytes = ByteBuffer.allocate(totalLength);
  // request packet format: | header (12 bytes) | body |
  // header: | flag: 6d 42 64 54 | request id (4 bytes) | packet code (4 bytes) |
  bytes.putInt(cons.TAIR_PACKET_FLAG); // packet flag
  bytes.putInt(reqId); // request id
  // | packet code (4 bytes) | body length (4 bytes) | body |
  bytes.putInt(this.code); // 12

  bytes.putInt(bodyLength);

  // body
  bytes.putInt(groupBytes.length);
  bytes.put(groupBytes);
  bytes.putInt(hostBytes.length);
  bytes.put(hostBytes);
  bytes.putInt(this.port);
  return bytes.array();
};

DataNodeHeartBeat.unpack = function (body) {
  var bytes = ByteBuffer.wrap(body);
  var data = {
    group: bytes.getString(),
    host: bytes.getString(),
    port: bytes.getInt()
  };
  return data;
};

module.exports = DataNodeHeartBeat;
