/**!
 * node-tair - lib/packets/inc_dec.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * Authors:
 *   苏千 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var util = require('util');
var ByteBuffer = require('byte');
var BasePacket = require('./base');
var cons = require('../const');

function IncDec(key, value, transcoder, options) {
  BasePacket.call(this, cons.TAIR_REQ_INCDEC_PACKET, transcoder);

  this.key = key;
  this.count = value;
  this.initValue = options.defaultValue;
  this.expired = options.expired;
  this.namespace = options.namespace;

  this.keyBytes = this.transcoder.encode(this.key);
}

util.inherits(IncDec, BasePacket);

IncDec.prototype.encode = function (reqId) {
  // format: | 0 (1 byte, body start) | namespace (2 bytes) | count (4 bytes) = 7
  //         | initValue (4 bytes) | expired (4 bytes) = 8
  //         | key meta flags (36 bytes)
  //         | key length (4 bytes) | key bytes
  var bodyLength = 55 + this.keyBytes.length;

  var totalLength = cons.PACKET_HEADER_SIZE + 4 + bodyLength;
  var bytes = ByteBuffer.allocate(totalLength);
  // request packet format: | header (12 bytes) | body length (4 bytes) | body (55 bytes) |
  // header: | flag: 6d 42 64 54 | request id (4 bytes) | packet code (4 bytes) |
  bytes.putInt(cons.TAIR_PACKET_FLAG); // packet flag
  bytes.putInt(reqId); // request id
  // | packet code (4 bytes) | body length (4 bytes) | body |
  bytes.putInt(this.code); // 12

  bytes.putInt(bodyLength);

  // body
  bytes.put(0);
  bytes.putShort(this.namespace);
  bytes.putInt(this.count);
  bytes.putInt(this.initValue);
  bytes.putInt(this.expired);

  // key
  this.fillMetaFlag(bytes);
  bytes.putInt(this.keyBytes.length);
  bytes.put(this.keyBytes);

  return bytes.array();
};

IncDec.prototype.decode = function (buf) {
  var res = {
    configVersion: buf.getInt(),
    value: buf.getInt(),
  };
  return res;
};

module.exports = IncDec;
