/**!
 * node-tair - lib/packets/lock.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * Authors:
 *   苏千 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var debug = require('debug')('tair:packets:lock');
var util = require('util');
var ByteBuffer = require('byte');
var cons = require('../const');
var BasePacket = require('./base');

/**
 * Lock packet
 *
 * @param {String|Buffer|Number} key
 * @package {Number} lockType
 * @param {Transcoder} transcoder
 * @param {Object} options
 *  - {Number} [namespace] defautl is 0
 */
function Lock(key, lockType, transcoder, options) {
  BasePacket.call(this, cons.TAIR_REQ_LOCK_PACKET, false);

  this.key = key;
  this.lockType = lockType;

  options = options || {};
  this.namespace = options.namespace || 0;
  this.transcoder = transcoder;

  this.keyBytes = this.transcoder.encode(this.key);
}

Lock.LOCK_STATUS = 1;
Lock.LOCK_VALUE = 2;
Lock.UNLOCK_VALUE = 3;
Lock.PUT_AND_LOCK_VALUE = 4;

util.inherits(Lock, BasePacket);

Lock.prototype.encode = function (reqId) {
  var keyBytes = this.keyBytes;

  // format: | namespace (2 bytes) | lockType (4 bytes)
  //         | key meta flags (36 bytes)
  //         | key length (4 bytes, CString) | key bytes
  var bodyLength = 46 + keyBytes.length;

  var totalLength = cons.PACKET_HEADER_SIZE + 4 + bodyLength;
  var bytes = ByteBuffer.allocate(totalLength);
  // request packet format: | header (12 bytes) | body |
  // header: | flag: 6d 42 64 54 | request id (4 bytes) | packet code (4 bytes) |
  bytes.putInt(cons.TAIR_PACKET_FLAG); // packet flag
  bytes.putInt(reqId); // request id
  // | packet code (4 bytes) | body length (4 bytes) | body |
  bytes.putInt(this.code); // 12

  bytes.putInt(bodyLength);

  // body
  bytes.putShort(this.namespace);
  bytes.putInt(this.lockType);

  // key
  this.fillMetaFlag(bytes, this.keyFlag);
  bytes.putInt(keyBytes.length);
  bytes.put(keyBytes);
  return bytes.array();
};

module.exports = Lock;
