/*!
 * node-tair - lib/packets/mixed_key.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   fengmk2 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var TairConstant = require('../const');

function MixedKey(transcoder, pkey, skey) {
  this.transcoder = transcoder;
  this.pkey = pkey;
  this.skey = skey;
  this.prefixSize = 0;
  this.encoded = false;
  this.pkeyBytes = null;
  this.skeyBytes = null;
  this.mixedBytes = null;
}

MixedKey.prototype.encode = function (isPrefix) {
  if (this.encoded) {
    return isPrefix ? this.pkeyBytes : this.mixedBytes;
  }
  this.pkeyBytes = this.transcoder.encode(this.pkey);

  var len = this.pkeyBytes.length;
  var bufs = [this.pkeyBytes];
  if (this.skey !== null) {
    this.skeyBytes = this.transcoder.encode(this.skey);
    len += this.skeyBytes.length;
    bufs.push(this.skeyBytes);
  }

  this.mixedBytes = Buffer.concat(bufs, len);

  var fg = new Buffer(2);
  var flag = TairConstant.TAIR_STYPE_MIXEDKEY;
  flag <<= 1;
  fg[1] = flag & 0xFF;
  fg[0] = (flag >> 8) & 0xFF;

  this.prefixSize = 2 + this.pkeyBytes.length;
  this.pkeyBytes = Buffer.concat([fg, this.pkeyBytes], this.prefixSize);

  this.encoded = true;
  if (isPrefix) {
    return this.pkeyBytes;
  }

  return this.mixedBytes;
};

MixedKey.prototype.decode = function (bytes, prefixSize) {
  prefixSize -= 2;
  this.pkey = this.transcoder.decode(bytes, 0, prefixSize);
  this.skey = this.transcoder.decode(bytes, prefixSize, bytes.length - prefixSize);
  return this;
};

module.exports = MixedKey;
