/*!
 * node-tair - lib/packets/ping.js
 *
 * Copyright(c) 2013 fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 * MIT Licensed
 */

"use strict";

/**
 * Module dependencies.
 */

var util = require('util');
var ByteBuffer = require('byte');
var cons = require('../const');
var BasePacket = require('./base');

function Ping(version, value) {
  BasePacket.call(this, cons.TAIR_REQ_PING_PACKET);

  this.version = version || 0;
  this.value = value || 0;
}

util.inherits(Ping, BasePacket);

Ping.prototype.encode = function (reqId) {
  var bodyLength = 8;
  var totalLength = cons.PACKET_HEADER_SIZE + 4 + bodyLength;
  var bytes = ByteBuffer.allocate(totalLength);
  // request packet format: | header (12 bytes) | body |
  // header: | flag: 6d 42 64 54 | request id (4 bytes) | packet code (4 bytes) |
  bytes.putInt(cons.TAIR_PACKET_FLAG); // packet flag
  bytes.putInt(reqId); // request id
  // | packet code (4 bytes) | body length (4 bytes) | body |
  bytes.putInt(this.code); // 12

  bytes.putInt(bodyLength);

  bytes.putInt(this.version);
  bytes.putInt(this.value);
  return bytes.array();
};

Ping.unpack = function (body) {
  var bytes = ByteBuffer.wrap(body);
  var data = {
    version: bytes.getInt(),
    value: bytes.getInt()
  };
  return data;
};

module.exports = Ping;
