/*!
 * node-tair - lib/packets/prefix_get.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   fengmk2 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var util = require('util');
var Get = require('./get');
var cons = require('../const');

/**
 * Prefix get packet
 *
 * @param {Array} keys
 * @param {Transcoder} transcoder
 * @param {Object} [options]
 *  - {Number} [namespace] default is 0
 */
function PrefixGet(keys, transcoder, options) {
  Get.call(this, keys[0], transcoder, options);
  this.code = cons.TAIR_REQ_PREFIX_GETS_PACKET;

  for (var i = 1; i < keys.length; i++) {
    this.addKey(keys[i]);
  }
}

util.inherits(PrefixGet, Get);

PrefixGet.prototype.decode = function (buf) {
  var res = {
    version: 0,
    code: 0,
    originalCode: 0,
    successCount: 0,
    failedCount: 0,
    key: null,
    value: null,
    codes: {}
  };

  res.version = buf.getInt();
  res.originalCode = res.code = buf.getInt();

  if (res.code === -3998) {
    res.code = 1;
  } else if (res.code === -3983) {
    res.code = 0;
  }

  // buf.skip(36);
  buf.position(buf.position() + 36);

  // pkey
  var size = buf.getInt();
  if (size > 0) {
    //~ omit the two leading bytes indicating the MixedKey type
    res.key = this.transcoder.decode(buf.read(size), 2, size - 2);
  }

  // success
  res.successCount = buf.getInt();

  var key = null;
  var value = null;

  if (res.successCount > 0) {
    res.value = {};

    for (var i = 0; i < res.successCount; i++) {
      // buf.skip(36);
      buf.position(buf.position() + 36);
      size = buf.getInt();
      if (size > 0) {
        key = this.transcoder.decode(buf.read(size), 0, size);
      }

      // buf.skip(36);
      buf.position(buf.position() + 36);
      size = buf.getInt();
      if (size > 0) {
        value = this.transcoder.decode(buf.read(size), 0, size);
      }
      var code = buf.getInt();
      res.codes[key] = code;
      res.value[key] = value;
    }
  }

  res.failedCount = buf.getInt();
  key = null;
  if (res.failedCount > 0) {
    for (var i = 0; i < res.failedCount; i++) {
      buf.skip(36);
      size = buf.getInt();
      if (size > 0) {
        key = this.transcoder.decode(buf.read(size), 0, size);
      }
      var code = buf.getInt();
      if (key) {
        res.codes[key] = code;
      }
    }
  }
  return res;
};

module.exports = PrefixGet;
