/**!
 * node-tair - lib/servers/config_server.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * Authors:
 *   苏千 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var debug = require('debug')('tair:servers:config_server');
var util = require('util');
var BaseServer = require('./base_server');

/**
 * Config Server
 * @param {Object} options
 *  - {Number} heartbeatInterval data heartbeat interval, default 10000 ms
 */
function ConfigServer(options) {
  BaseServer.call(this, options);
  options = options || {};
  this._heartbeatInterval = options.heartbeatInterval || 10000;
  this._heartbeatInterval += 5000;

  // {groupname: servers}
  this._groups = {};

  this.on('request', function (req, res) {
    if (req.name === 'get_group') {
      return this._handleGetGroup(req, res);
    } else if (req.name === 'data_node_hearbeat') {
      return this._handleDataNodeHeartbeat(req, res);
    } else {
      return res.sendError({message: 'unsupport request, code ' + req.code, code: -20008});
    }
  }.bind(this));
}

util.inherits(ConfigServer, BaseServer);

ConfigServer.prototype._handleGetGroup = function (req, res) {
  var servers = this._groups[req.data.name] || [];
  var now = Date.now();
  var heartbeatInterval = this._heartbeatInterval;
  servers = servers.filter(function (server) {
    // _heartbeatInterval: got a heartbeat
    return now - server.timestamp < heartbeatInterval;
  });
  this._groups[req.data.name] = servers;

  var data = {
    bucket: servers.length,
    copy: 1,
    version: servers.length === 0 ? 0 : 1, // 0: not exists
    configCount: 0,
    servers: servers
  };
  res.send(data);
};

ConfigServer.prototype._handleDataNodeHeartbeat = function (req, res) {
  var server = {host: req.data.host, port: req.data.port, timestamp: Date.now()};
  var group = req.data.group;
  if (!this._groups[group]) {
    this._groups[group] = [];
  }
  var servers = this._groups[group];
  var found = false;
  for (var i = 0; i < servers.length; i++) {
    var s = servers[i];
    if (s.host === server.host && s.port === server.port) {
      found = true;
      s.timestamp = server.timestamp;
      break;
    }
  }
  if (!found) {
    servers.push(server);
  }
  debug('data node heartbeat, group: %s has %d servers', group, servers.length);
  res.send({version: 0, value: 0});
};

module.exports = ConfigServer;
