/**!
 * node-tair - lib/servers/middlewares/accesslog.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * Authors:
 *   苏千 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var qs = require('qs');
var VERSION = 'tair/' + require('../../../package.json').version;

/**
 * Access log middleware
 *
 * @param {Object} options
 *  - {Object} logger, must support `accessLog(ip, usec, method, url, status, bytes, referer, userAgent, hitCache, backendTime)`
 *  - {Object} [timer] default is `Date`
 *  - {String} [loggerMethod] default is 'accessLog',
 *      if you want to log backend time, you need to set 'accessLogExtra'
 * @return {Function(res)}
 */
module.exports = function (options) {
  options.loggerMethod = options.loggerMethod || 'accessLog';
  options.timer = options.timer || Date;
  return function accessLogMiddleware(req, res, next) {
    // TODO: handle Buffer data
    var method = req.name.toUpperCase();
    var data = req.data || {};
    var namespace = data.namespace || -1;
    var start = options.timer.now();

    res.__endOld = res.end;
    res.end = function (bytes, extras) {
      if (this.ended) {
        return;
      }

      var size = this.__endOld(bytes, extras);

      var querystring = '';
      try {
        querystring = qs.stringify(data);
      } catch (e) {
        querystring = qs.stringify({key: data.key, namespace: namespace});
        console.error('[%s] %s, error: %s, data: %j', Date(), req.name, e, data);
      }

      var url = '/tair/' + namespace + '/' + req.name + '?' + querystring;
      var use = options.timer.now() - start;
      var status = 200;

      var backendTime = 0;
      var hitCache = 0;
      if (extras) {
        backendTime = extras.use || 0; // 访问后端数据源耗时
        hitCache = extras.cache ? 1 : 0; // 是否命中cache
        status = extras.status || status;
      }

      var remoteAddress = req.remoteAddress || '-';
      options.logger[options.loggerMethod](remoteAddress, use, method, url,
        status, size, '-', VERSION, hitCache, backendTime);
      return size;
    };

    next();
  };
};
