/**!
 * node-tair - lib/servers/response.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * Authors:
 *   苏千 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var debug = require('debug')('tair:servers:response');
var ByteBuffer = require('byte');
var Codes = require('./codes');
var cons = require('../const');
var DefaultPacket = require('../packets').Base;

function TairResponse(req, transcoder, socket) {
  this.req = req;
  this.socket = socket;
  this.transcoder = transcoder;

  var pkg = Codes[this.req.code];
  this.pkg = pkg && pkg.class || DefaultPacket;
  this.responseCode = this.pkg.ResponseCode || DefaultPacket.ResponseCode;
  this.ended = false;
}

TairResponse.prototype.end = function (bytes, extras) {
  if (this.ended) {
    return;
  }
  this.ended = true;
  this.socket.write(bytes);
  return bytes.length;
};

TairResponse.prototype.send = function (value, extras) {
  var pkg = this.pkg;
  debug('send req#%s response, code: %s, extras: %j', this.req.id, this.responseCode, extras);
  var data = value;
  var t = typeof value;
  if (Buffer.isBuffer(value) || t === 'string' || t === 'number' || t === 'boolean') {
    // not Object or Array
    data = {value: value};
  }
  extras = extras || {};
  extras.status = data.status || extras.status || 200;

  var size = 0;
  if (pkg.asyncPack) {
    var that = this;
    pkg.asyncPack(data, that.transcoder, function (err, body) {
      if (err) {
        return that.sendError(err.message, -3999);
      }

      var packetSize = 16 + body.length;
      var bytes = ByteBuffer.allocate(packetSize);
      bytes.putInt(cons.TAIR_PACKET_FLAG);
      bytes.putInt(that.req.id);
      bytes.putInt(that.responseCode);
      bytes.putInt(body.length);
      bytes.put(body);
      that.end(bytes.array(), extras);
    });
  } else {
    if (!pkg.pack) {
      pkg = DefaultPacket;
    }
    var body = pkg.pack(data, this.transcoder);
    var packetSize = 16 + body.length;
    var bytes = ByteBuffer.allocate(packetSize);
    bytes.putInt(cons.TAIR_PACKET_FLAG);
    bytes.putInt(this.req.id);
    bytes.putInt(this.responseCode);
    bytes.putInt(body.length);
    bytes.put(body);
    size = this.end(bytes.array(), extras);
  }
  return size;
};

TairResponse.prototype.sendError = function (err, extras) {
  extras = extras || {};
  extras.status = err.status || extras.status || 500;

  var code = err.code || -2;
  var bytes = ByteBuffer.allocate(100);
  bytes.putInt(cons.TAIR_PACKET_FLAG);
  bytes.putInt(this.req.id);
  bytes.putInt(this.responseCode);
  var pkg = this.pkg;
  if (!pkg.packError) {
    pkg = DefaultPacket;
  }
  var body = pkg.packError({version: 0, value: code, message: err.message}, this.transcoder);
  bytes.putInt(body.length);
  bytes.put(body);
  return this.end(bytes.array(), extras);
};

module.exports = TairResponse;
