/*!
 * tair - lib/utils.js
 *
 * Copyright(c) 2012 - 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   kate.sf <kate.sf@taobao.com>
 *   fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 */

/**
 * utility module
 */

exports.noop = function () {};

exports.longToHost = function (id) {
  var host = '';
  host += ((id & 0xff)).toString() + '.';
  host += (((id >> 8) & 0xff)).toString() + '.';
  host += (((id >> 16) & 0xff)).toString() + '.';
  host += (((id >> 24) & 0xff)).toString();
  return host;
};

exports.hostToLong = function (host) {
  var items = host.split('.');
  var ipa = 0;
  ipa |= ((Number(items[3]) << 24) & 0xff000000);
  ipa |= ((Number(items[2]) << 16) & 0xff0000);
  ipa |= ((Number(items[1]) << 8) & 0xff00);
  ipa |= (Number(items[0]) & 0xff);
  return ipa;
};

var MURMURHASH_M = 0x5bd1e995;
exports.murmurhash2 = function murmurhash2(key) {
  var l = key.length;
  var h = 97 ^ l;
  var i = 0;
  var k = null;

  while (l >= 4) {
    k = ((key[i] & 0xff)) |
      ((key[++i] & 0xff) << 8) |
      ((key[++i] & 0xff) << 16) |
      ((key[++i] & 0xff) << 24);

    k = (((k & 0xffff) * MURMURHASH_M) + ((((k >>> 16) * MURMURHASH_M) & 0xffff) << 16));
    k ^= k >>> 24;
    k = (((k & 0xffff) * MURMURHASH_M) + ((((k >>> 16) * MURMURHASH_M) & 0xffff) << 16));

    h = (((h & 0xffff) * MURMURHASH_M) + ((((h >>> 16) * MURMURHASH_M) & 0xffff) << 16)) ^ k;

    l -= 4;
    ++i;
  }

  switch (l) {
    case 3:
      h ^= (key[i + 2] & 0xff) << 16;
    case 2:
      h ^= (key[i + 1] & 0xff) << 8;
    case 1:
      h ^= (key[i] & 0xff);
      h = (((h & 0xffff) * MURMURHASH_M) + ((((h >>> 16) * MURMURHASH_M) & 0xffff) << 16));
  }

  h ^= h >>> 13;
  h = (((h & 0xffff) * MURMURHASH_M) + ((((h >>> 16) * MURMURHASH_M) & 0xffff) << 16));
  h ^= h >>> 15;

  return h >>> 0;
};

exports.printBuffer = function (buf) {
  var s = '<Buffer';
  for (var i = 0; i < buf.length; i++) {
    s += ' ' + buf[i].toString(16);
  }
  console.log(s + '>');
};
