/*!
 * node-tair - test/client.test.js
 *
 * Copyright(c) 2012 - 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   fengmk2 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 *   sunfang1cn <kate.sf@taobao.com>
 */

"use strict";

/**
 * Module dependencies.
 */

var fs = require('fs');
var path = require('path');
var Tair = require('../..');

var fixtures = __dirname;

var tair = new Tair({
  dataId: 'ldbcommon-daily',
  diamond: 'commonconfig.taobao.net:8080',
  namespace: 171,
  debug: true,
}).on('connect', get100Bytes);

function bytes(size) {
  var buf = new Buffer(size);
  for (var i = 0; i < buf.length; i++) {
    buf[i] = 97; // a
  }
  return buf.toString();
}

function getBytes(size, callback) {
  var value = bytes(size);
  var key = 'nodejs-fixtures-' + size + 'bytes';
  tair.put(key, value, function (err, result) {
    if (err) {
      throw err;
    }
    console.log('put %dbytes: %s', size, result);
    tair.get(key, function (err, val, res) {
      if (err) {
        throw err;
      }
      console.log('get %dbytes: %s, value size: %d, body size: %d',
        size,
        val === value, val.length, res._body.length);
      fs.writeFileSync(path.join(fixtures, 'get_' + size + 'bytes_response.bin'), res._body);
      callback();
    });
  });
}

function prefixGetsBytes(size, callback) {
  var value = bytes(size);
  var key = 'nodejs-fixtures-prefix-' + size + 'bytes';
  var k1 = 'user_balabala_hahaa_fb';
  var k2 = 'balabala_hahaa_fb';
  var values = [
    [k1, value],
    [k2, value],
  ];
  tair.prefixPuts(key, values, function (err, result) {
    if (err) {
      throw err;
    }
    console.log('prefixPuts %dbytes: %s', size, result);
    tair.prefixGets(key, [k1, k2], function (err, obj, res) {
      if (err) {
        throw err;
      }
      console.log('prefixGets %dbytes: %s, value size: %d, body size: %d',
        size,
        obj[k1] === value && obj[k2] === value,
        obj[k1].length + obj[k2].length,
        res._body.length);
      fs.writeFileSync(path.join(fixtures, 'prefix_gets_' + size + 'bytes_response.bin'), res._body);
      callback();
    });
  });
}

function get100Bytes() {
  getBytes(100, function () {
    get500Bytes();
  });
}

function get500Bytes() {
  getBytes(500, function () {
    get1000Bytes();
  });
}

function get1000Bytes() {
  getBytes(1000, function () {
    prefixGets100Bytes();
  });
}

function prefixGets100Bytes() {
  prefixGetsBytes(100, function () {
    prefixGets500Bytes();
  });
}

function prefixGets500Bytes() {
  prefixGetsBytes(500, function () {
    prefixGets1000Bytes();
  });
}

function prefixGets1000Bytes() {
  prefixGetsBytes(1000, function () {
    process.exit(0);
  });
}

