/*!
 * node-tair - test/packets/get_range.test.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   fengmk2 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var should = require('should');
var GetRange = require('../../lib/packets/get_range');
var Transcoder = require('../../lib/packets/transcoder');
var MixedKey = require('../../lib/packets/mixed_key');
var cons = require('../../lib/const');

describe('packets/get_range.test.js', function () {
  it('should encode get_range bytes match java bytes', function () {
    var bodyLength = 143;
    var pkey = '320320320';
    var start = '8616882133';
    var end = '8616882233';
    var chid = 2;
    var namespace = 171;
    var javaBytes = new Buffer([
      109, 66, 100, 84, 0, 0, 0, 2,
      0, 0, 0, 18,
      0, 0, 0, -113,
      0, 0, 2, 0, -85, 0, 0, 0, 0, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 64, 0, 25, 0, 24, 0, 4, 51, 50, 48, 51, 50, 48, 51, 50, 48, 0, 4, 56, 54, 49, 54, 56, 56, 50, 49, 51, 51, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 64, 0, 25, 0, 24, 0, 4, 51, 50, 48, 51, 50, 48, 51, 50, 48, 0, 4, 56, 54, 49, 54, 56, 56, 50, 50, 51, 51]);

    var transcoder = new Transcoder();
    var startKey = new MixedKey(transcoder, pkey, start);
    var endKey = new MixedKey(transcoder, pkey, end);
    var getRange = new GetRange(startKey, endKey, 2, 0, 100, transcoder, {namespace: 171});

    var b = getRange.encode(chid);
    b.length.should.equal(javaBytes.length);
    b.should.eql(javaBytes);
    // b.slice(0, 143).should.eql(javaBytes.slice(0, 143));
  });
});
