/**!
 * node-tair - test/servers/request.test.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * Authors:
 *   苏千 <suqian.yf@taobao.com> (http://fengmk2.github.com)
 */

"use strict";

/**
 * Module dependencies.
 */

var should = require('should');
var Bytes = require('byte');
var Request = require('../../lib/servers/request');
var Response = require('../../lib/servers/response');
var packets = require('../../lib/packets');
var Transcoder = require('../../lib/packets/transcoder');
var cons = require('../../lib/const');

describe('servers/request.test.js', function () {
  it('should unpack a get packet withHeader=false', function (done) {
    var cppTranscoder = new Transcoder(false);
    var get = new packets.Get('foo', cppTranscoder, {namespace: 171});
    var data = get.encode(2);
    var bytes = Bytes.wrap(data);

    var flag = bytes.getInt();
    var reqId = bytes.getInt();
    var packetCode = bytes.getInt();
    packetCode.should.equal(cons.TAIR_REQ_GET_PACKET);
    var length = bytes.getInt();
    var body = bytes.read(length);
    var req = new Request(reqId, packetCode, body, cppTranscoder, {});
    req.id.should.equal(2);
    req.code.should.equal(cons.TAIR_REQ_GET_PACKET);
    req.data.namespace.should.equal(171);
    req.data.key.should.eql('foo');

    var res = new Response(req, cppTranscoder, {
      write: function (buf) {
        var buf = Bytes.wrap(buf);
        buf.getInt().should.equal(cons.TAIR_PACKET_FLAG);
        buf.getInt().should.equal(2);
        buf.getInt().should.equal(packets.Get.ResponseCode);
        buf.getInt().should.above(0);
        var data = get.decode(buf);
        data.should.eql({
          version: 0,
          code: 0,
          count: 1,
          key: null,
          value: 'bar',
          meta: { isMerged: false, area: 0, serverFlag: 0 }
        });
        done();
      }
    });
    res.send('bar');
  });

  it('should unpack a get packet withHeader=true', function (done) {
    var transcoder = new Transcoder();
    var get = new packets.Get('foo', transcoder, {namespace: 171});
    var data = get.encode(3);
    var bytes = Bytes.wrap(data);

    var flag = bytes.getInt();
    var reqId = bytes.getInt();
    var packetCode = bytes.getInt();
    packetCode.should.equal(cons.TAIR_REQ_GET_PACKET);
    var length = bytes.getInt();
    var body = bytes.read(length);
    var req = new Request(reqId, packetCode, body, transcoder, {});
    req.id.should.equal(3);
    req.code.should.equal(cons.TAIR_REQ_GET_PACKET);
    req.data.namespace.should.equal(171);
    req.data.key.should.eql('foo');

    var res = new Response(req, transcoder, {
      write: function (buf) {
        var buf = Bytes.wrap(buf);
        buf.getInt().should.equal(cons.TAIR_PACKET_FLAG);
        buf.getInt().should.equal(3);
        buf.getInt().should.equal(packets.Get.ResponseCode);
        buf.getInt().should.above(0);
        var data = get.decode(buf);
        data.should.eql({
          version: 0,
          code: 0,
          count: 1,
          key: null,
          value: 'bar',
          meta: { isMerged: false, area: 0, serverFlag: 0 }
        });
        done();
      }
    });
    res.send('bar');
  });
});
