/**!
 * node-tair - test/utils.test.js
 *
 * Copyright(c) 2013 Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   kate.sf <kate.sf@taobao.com>
 *   fengmk2 <fengmk2@gmail.com> (http://fengmk2.github.com)
 */

var utils = require('../lib/utils');
var Transcoder = require('../lib/packets/transcoder');
var MixedKey = require('../lib/packets/mixed_key');

describe('utils.test.js', function () {
  describe('longToHost(), hostToLong()', function () {
    it('should convert host to long and long to host', function () {
      utils.hostToLong('127.0.0.1').should.equal(16777343);
      utils.longToHost(16777343).should.equal('127.0.0.1');
      utils.longToHost(utils.hostToLong('10.1.23.254')).should.equal('10.1.23.254');
    });
  });

  describe('murmurhash2()', function () {
    it('should convert "node-tair-foo-java" get key hash match java', function () {
      // 426136330:
      var b = new Buffer([0, 4, 110, 111, 100, 101, 45, 116, 97, 105, 114, 45, 102, 111, 111, 45, 106, 97, 118, 97]);
      utils.murmurhash2(b).should.equal(426136330);
    });

    it('should hash prefix:435674456 got: 2351022266', function () {
      var key = new MixedKey(new Transcoder(), '435674456', '');
      var keybytes = key.encode(true);
      var wantBytes = new Buffer([0, 24, 0, 4, 52, 51, 53, 54, 55, 52, 52, 53, 54]);
      keybytes.should.eql(wantBytes);
      // console.log(wantBytes, new Buffer('435674456'))
      // console.log(keybytes, wantBytes)
      utils.murmurhash2(keybytes).should.equal(2351022266);
      parseInt(utils.murmurhash2(keybytes) % 1023, 10).should.equal(494);
    });
  });
});
